@@ -1,8 +1,9 @@
 netgraph <- function(x, seq = x$seq,
-                     labels = rownames(x$TE.fixed),
-                     cex = 1, col = "slateblue", offset = 0.0175,
+                     labels = x$trts,
+                     cex = 1, adj = NULL,
+                     offset = if (!is.null(adj) && all(unique(adj) == 0.5)) 0 else 0.0175,
                      scale = 1.10,
-                     plastic, thickness,
+                     col = "slateblue", plastic, thickness,
                      lwd = 5, lwd.min = lwd / 2.5, lwd.max = lwd * 4,
                      dim = "2d",
                      ##
@@ -20,6 +21,7 @@ netgraph <- function(x, seq = x$seq,
                      cex.number.of.studies = cex,
                      col.number.of.studies = "white",
                      bg.number.of.studies = "black",
+                     pos.number.of.studies = 0.5,
                      ##
                      start.layout = ifelse(dim == "2d", "circle", "eigen"),
                      eig1 = 2, eig2 = 3, eig3 = 4,
@@ -34,9 +36,23 @@ netgraph <- function(x, seq = x$seq,
   
   
   meta:::chkclass(x, "netmeta")
+  ##
+  n.edges <- sum(x$A.matrix[upper.tri(x$A.matrix)] > 0)
+  n.trts <- length(x$trts)
   
   
-  dim <- meta:::setchar(dim, c("2d", "3d"))
+  setchar <- meta:::setchar
+  chknumeric <- meta:::chknumeric
+  ##
+  chknumeric(lwd, min = 0, zero = TRUE, single = TRUE)
+  chknumeric(lwd.min, min = 0, zero = TRUE, single = TRUE)
+  chknumeric(lwd.max, min = 0, zero = TRUE, single = TRUE)
+  chknumeric(lwd.highlight, min = 0, zero = TRUE, single = TRUE)
+  ##
+  if (lwd.min > lwd.max)
+    stop("Argument 'lwd.min' must be smaller than 'lwd.max'.")
+  ##
+  dim <- setchar(dim, c("2d", "3d"))
   is_2d <- dim == "2d"
   is_3d <- !is_2d
   ##
@@ -54,13 +70,65 @@ netgraph <- function(x, seq = x$seq,
     is_3d <- FALSE
   }
   ##
-  start.layout <- meta:::setchar(start.layout, c("eigen", "prcomp", "circle", "random"))
+  start.layout <- setchar(start.layout, c("eigen", "prcomp", "circle", "random"))
   ##
   if (!missing(seq) & is.null(seq))
     stop("Argument 'seq' must be not NULL.")
   ##
   if (!missing(labels) & is.null(labels))
     stop("Argument 'labels' must be not NULL.")
+  ##
+  ## Colors of edges
+  ##
+  if (is.matrix(col)) {
+    if ((dim(col)[1] != dim(A.matrix)[1]) |
+        (dim(col)[2] != dim(A.matrix)[2]))
+      stop("Dimension of argument 'A.matrix' and 'col' are different.")
+    if (is.null(dimnames(col)))
+      stop("Matrix 'col' must have row and column names identical to argument 'A.matrix'.")
+    else {
+      if (any(rownames(col) != rownames(A.matrix)))
+        stop("Row names of matrix 'col' must be identical to argument 'A.matrix'.")
+      if (any(colnames(col) != colnames(A.matrix)))
+        stop("Column names of matrix 'col' must be identical to argument 'A.matrix'.")
+    }
+    ##
+    col <- col[lower.tri(col)]
+    col <- col[!is.na(col)]
+  }
+  ##
+  n.col <- length(col)
+  ##
+  if (n.col == 1)
+    col <- rep(col, n.edges)
+  else if (n.col != n.edges)
+    stop("Length of argument 'col' (",
+         n.col, ") is different from the number of direct pairwise comparisons (",
+         n.edges, ")")
+  ##
+  n.pos <- length(pos.number.of.studies)
+  if (n.pos == 1)
+    pos.number.of.studies <- rep(pos.number.of.studies, n.edges)
+  else if (n.pos != n.edges)
+    stop("Length of argument 'pos.number.of.studies' (",
+         n.pos, ") is different from the number of direct pairwise comparisons (",
+         n.edges, ")")
+  ##
+  if (length(cex.points) == 1)
+    cex.points <- rep(cex.points, n.trts)
+  else if (length(cex.points) != n.trts)
+    stop("Length of argument 'cex.points' must be equal to the number of treatments.")
+  ##
+  if (length(col.points) == 1)
+    col.points <- rep(col.points, n.trts)
+  else if (length(col.points) != n.trts)
+    stop("Length of argument 'col.points' must be equal to the number of treatments.")
+  ##
+  if (length(pch.points) == 1)
+    pch.points <- rep(pch.points, n.trts)
+  else if (length(pch.points) != n.trts)
+    stop("Length of argument 'pch.points' must be equal to number of treatments.")
+  
   
   if (missing(iterate))
     iterate <- ifelse(start.layout == "circle", FALSE, TRUE)
@@ -97,9 +165,9 @@ netgraph <- function(x, seq = x$seq,
   else {
     if (!is.matrix(thickness)) {
       if (length(thickness) == 1 & is.character(thickness))
-        thick <- meta:::setchar(thickness,
-                                c("equal", "number.of.studies",
-                                  "se.fixed", "se.random", "w.fixed", "w.random"))
+        thick <- setchar(thickness,
+                         c("equal", "number.of.studies",
+                           "se.fixed", "se.random", "w.fixed", "w.random"))
       ##
       else if (length(thickness) == 1 & is.logical(thickness)) {
         if (thickness)
@@ -143,14 +211,37 @@ netgraph <- function(x, seq = x$seq,
     seq1 <- charmatch(setseq(seq, rn), rn)
   }
   ##
+  col.matrix <- matrix("", nrow = n.trts, ncol = n.trts)
+  dimnames(col.matrix) <- dimnames(A.matrix)
+  col.matrix <- t(col.matrix)
+  col.matrix[lower.tri(col.matrix) & t(A.matrix) > 0] <- col
+  tcm <- col.matrix
+  col.matrix <- t(col.matrix)
+  col.matrix[lower.tri(col.matrix)] <- tcm[lower.tri(tcm)]
+  ##
+  pos.matrix <- matrix(NA, nrow = n.trts, ncol = n.trts)
+  dimnames(pos.matrix) <- dimnames(A.matrix)
+  pos.matrix <- t(pos.matrix)
+  pos.matrix[lower.tri(pos.matrix) & t(A.matrix) > 0] <- pos.number.of.studies
+  tam <- pos.matrix
+  pos.matrix <- t(pos.matrix)
+  pos.matrix[lower.tri(pos.matrix)] <- tam[lower.tri(tam)]
+  ##
   A.matrix <- A.matrix[seq1, seq1]
   N.matrix <- N.matrix[seq1, seq1]
   D.matrix <- D.matrix[seq1, seq1]
   ##
+  col.matrix <- col.matrix[seq1, seq1]
+  pos.matrix <- pos.matrix[seq1, seq1]
+  ##
   if (thick == "matrix")
     W.matrix <- W.matrix[seq1, seq1]
   ##
   labels <- labels[seq1]
+  ##
+  col.points <- col.points[seq1]
+  cex.points <- cex.points[seq1]
+  pch.points <- pch.points[seq1]
   
   
   A.sign <- sign(A.matrix)
@@ -177,6 +268,7 @@ netgraph <- function(x, seq = x$seq,
                          labels = labels,
                          cex = cex,
                          col = col,
+                         adj = adj,
                          offset = offset,
                          scale = scale,
                          ##
@@ -200,6 +292,7 @@ netgraph <- function(x, seq = x$seq,
                          cex.number.of.studies = cex.number.of.studies,
                          col.number.of.studies = col.number.of.studies,
                          bg.number.of.studies = bg.number.of.studies,
+                         pos.number.of.studies = pos.number.of.studies,
                          ##
                          ...)
     ##
@@ -218,39 +311,156 @@ netgraph <- function(x, seq = x$seq,
   d <- scale * max(abs(c(min(c(xpos, ypos), na.rm = TRUE),
                          max(c(xpos, ypos), na.rm = TRUE))))
   
-  
-  ## Generate dataset for plotting
+
   ##
+  ##
+  ## Generate datasets for plotting
+  ##
+  ##
+  ##
+  ## Dataset for nodes
+  ##
+  dat.nodes <- data.frame(labels, seq,
+                          xpos, ypos, zpos = NA,
+                          xpos.labels = NA, ypos.labels = NA,
+                          cex = cex.points,
+                          col = col.points,
+                          pch = pch.points,
+                          stringsAsFactors = FALSE)
   if (is_2d)
-    pd <- data.frame(xpos, ypos, labels, seq)
-  else
-    pd <- data.frame(xpos, ypos, zpos, labels, seq)
-  ##
-  pd$adj1 <- NA
-  pd$adj2 <- NA
-  pd$adj3 <- NA
-  ##
-  pd$adj1[pd$xpos >= 0] <- 0
-  pd$adj1[pd$xpos <  0] <- 1
-  ##
-  pd$adj2[pd$ypos >  0] <- 0
-  pd$adj2[pd$ypos <= 0] <- 1
-  ##
-  if (!is_2d) {
-    pd$adj3[pd$zpos >  0] <- 0
-    pd$adj3[pd$zpos <= 0] <- 1
+    dat.nodes$zpos <- NULL
+  else {
+    dat.nodes$zpos <- zpos
+    dat.nodes$zpos.labels <- NA
   }
   ##
-  offset <- offset * 2 * d
-  ##
-  if (is_2d) {
-    pd$xpos.labels <- pd$xpos - offset + 2 * (pd$adj1 == 0) * offset
-    pd$ypos.labels <- pd$ypos - offset + 2 * (pd$adj2 == 0) * offset
+  if (is.null(adj)) {
+    dat.nodes$adj.x <- NA
+    dat.nodes$adj.y <- NA
+    if (!is_2d)
+      dat.nodes$adj.z <- NA
+    ##
+    dat.nodes$adj.x[dat.nodes$xpos >= 0] <- 0
+    dat.nodes$adj.x[dat.nodes$xpos <  0] <- 1
+    ##
+    dat.nodes$adj.y[dat.nodes$ypos >  0] <- 0
+    dat.nodes$adj.y[dat.nodes$ypos <= 0] <- 1
+    ##
+    if (!is_2d) {
+      dat.nodes$adj.z[dat.nodes$zpos >  0] <- 0
+      dat.nodes$adj.z[dat.nodes$zpos <= 0] <- 1
+    }
   }
   else {
-    pd$xpos.labels <- pd$xpos
-    pd$ypos.labels <- pd$ypos
-    pd$zpos.labels <- pd$zpos
+    dat.nodes$adj.x <- NA
+    dat.nodes$adj.y <- NA
+    ##
+    if (length(adj) == 1) {
+      dat.nodes$adj.x <- adj
+      dat.nodes$adj.y <- adj
+      if (!is_2d)
+        dat.nodes$adj.z <- adj
+    }
+    else if (length(adj) == 2) {
+      dat.nodes$adj.x <- adj[1]
+      dat.nodes$adj.y <- adj[2]
+      if (!is_2d)
+        dat.nodes$adj.z <- 0.5
+    }
+    else if (length(adj) == 3 & !is_2d) {
+      dat.nodes$adj.x <- adj[1]
+      dat.nodes$adj.y <- adj[2]
+      dat.nodes$adj.z <- adj[3]
+    }
+    else if (is.vector(adj)) {
+        if (length(adj) != length(labels))
+          stop("Length of vector 'adj' must be equal to number of treatments.")
+        ##
+        names(adj) <- x$trts
+        dat.nodes$adj.x <- adj[seq1]
+        dat.nodes$adj.y <- adj[seq1]
+        ##
+        if (!is_2d)
+          dat.nodes$adj.z <- adj[seq1]
+    }
+    else if (is.matrix(adj)) {
+      if (nrow(adj) != length(labels))
+        stop("Number of rows of matrix 'adj' must be equal to number of treatments.")
+      rownames(adj) <- x$trts
+      dat.nodes$adj.x <- adj[seq1, 1]
+      dat.nodes$adj.y <- adj[seq1, 2]
+      ##
+      if (!is_2d & ncol(adj) >= 3)
+        dat.nodes$adj.z <- adj[seq1, 3]
+    }
+  }
+  ##
+  if (is_2d) {
+    offset <- offset * 2 * d
+    ##
+    if (length(offset) == 1) {
+      offset.x <- offset
+      offset.y <- offset
+    }
+    else if (length(offset) == 2) {
+      offset.x <- offset[1]
+      offset.y <- offset[2]
+    }
+    else if (is.vector(offset)) {
+      if (length(offset) != length(labels))
+        stop("Length of vector 'offset' must be equal to number of treatments.")
+      ##
+      rownames(offset) <- x$trts
+      offset.x <- offset[seq1]
+      offset.y <- offset[seq1]
+    }
+    else if (is.matrix(adj)) {
+      if (nrow(offset) != length(labels))
+        stop("Number of rows of matrix 'offset' must be equal to number of treatments.")
+      ##
+      rownames(offset) <- x$trts
+      offset.x <- offset[seq1, 1]
+      offset.y <- offset[seq1, 2]
+    }
+    ##
+    dat.nodes$xpos.labels <- dat.nodes$xpos - offset.x +
+      2 * (dat.nodes$adj.x == 0) * offset.x
+    dat.nodes$ypos.labels <- dat.nodes$ypos - offset.y +
+      2 * (dat.nodes$adj.y == 0) * offset.y
+  }
+  else {
+    dat.nodes$xpos.labels <- dat.nodes$xpos
+    dat.nodes$ypos.labels <- dat.nodes$ypos
+    dat.nodes$zpos.labels <- dat.nodes$zpos
+  }
+  ##
+  ## Dataset for edges
+  ##
+  dat.edges <- data.frame(treat1 = rep("", n.edges),
+                          treat2 = "",
+                          n.stud = NA,
+                          xpos = NA, ypos = NA,
+                          adj = NA, pos.number.of.studies,
+                          col = "",
+                          stringsAsFactors = FALSE)
+  ##
+  comp.i <- 1
+  ##
+  for (i in 1:(n - 1)) {
+    for (j in (i + 1):n) {
+      if (A.sign[i, j] > 0) {
+        ##
+        dat.edges$treat1[comp.i] <- rownames(A.matrix)[i]
+        dat.edges$treat2[comp.i] <- colnames(A.matrix)[j]
+        dat.edges$n.stud[comp.i] <- A.matrix[i, j]
+        dat.edges$adj[comp.i] <- lambda <- pos.matrix[i, j]
+        dat.edges$xpos[comp.i] <- lambda * xpos[i] + (1 - lambda) * xpos[j]
+        dat.edges$ypos[comp.i] <- lambda * ypos[i] + (1 - lambda) * ypos[j]
+        dat.edges$col[comp.i] <- col.matrix[i, j]
+        ##
+        comp.i <- comp.i + 1
+      }
+    }
   }
   
   
@@ -396,9 +606,9 @@ netgraph <- function(x, seq = x$seq,
           treat2 <- x$treat2[x$studlab %in% multiarm.studies[i]]
           multiarm.labels[[i]] <- sort(unique(c(treat2, treat1)))
           ##
-          pdm <- pd[pd$labels %in% multiarm.labels[[i]], ]
-          if (nrow(pdm) == 0)
-            pdm <- pd[pd$labels %in% multiarm.labels[[i]], ]
+          dat.multi <- dat.nodes[dat.nodes$labels %in% multiarm.labels[[i]], ]
+          if (nrow(dat.multi) == 0)
+            dat.multi <- dat.nodes[dat.nodes$labels %in% multiarm.labels[[i]], ]
           ##
           ## Clockwise ordering of polygon coordinates
           ##
@@ -416,9 +626,9 @@ netgraph <- function(x, seq = x$seq,
             res
           }
           ##
-          pdm <- pdm[polysort(pdm$xpos, pdm$ypos), ]
+          dat.multi <- dat.multi[polysort(dat.multi$xpos, dat.multi$ypos), ]
           ##
-          polygon(pdm$xpos, pdm$ypos,
+          polygon(dat.multi$xpos, dat.multi$ypos,
                   col = col.polygon[i], border = NA)
         }
       }
@@ -448,13 +658,23 @@ netgraph <- function(x, seq = x$seq,
       cols.highlight <- col.highlight
     }
     ##
+    comp.i <- 1
+    ##
     for (n.plines in 1:length(lwd.multiply)) {
       for (i in 1:(n - 1)) {
         for (j in (i + 1):n) {
+          ##
+          if (plastic)
+            col.ij <- cols[n.plines]
+          else
+            col.ij <- col.matrix[i, j]
+          ##
           if (A.sign[i, j] > 0) {
             lines(c(xpos[i], xpos[j]), c(ypos[i], ypos[j]),
                   lwd = W.matrix[i, j] * lwd.multiply[n.plines],
-                  col = cols[n.plines])
+                  col = col.ij)
+            ##
+            comp.i <- comp.i + 1
           }
         }
       }
@@ -464,20 +684,20 @@ netgraph <- function(x, seq = x$seq,
     ##
     if (!is.null(highlight)) {
       for (high in highlight) {
-        highs <- unlist(strsplit(high, split = highlight.split))
+        highs <- unlist(compsplit(high, split = highlight.split))
         if (length(highs) != 2)
           stop("Wrong format for argument 'highlight' (see helpfile of plotgraph command).")
         ##
-        if (sum(pd$labels %in% highs) != 2)
+        if (sum(dat.nodes$labels %in% highs) != 2)
           stop(paste("Argument 'highlight' must contain two of the following values (separated by \":\"):\n  ",
-                     paste(paste("'", pd$labels, "'", sep = ""),
+                     paste(paste("'", dat.nodes$labels, "'", sep = ""),
                            collapse = " - "), sep = ""))
         ##
-        pdh <- pd[pd$labels %in% highs, ]
+        dat.high <- dat.nodes[dat.nodes$labels %in% highs, ]
         ##
         if (is_2d)
           for (n.plines in 1:length(lwd.multiply))
-            lines(pdh$xpos, pdh$ypos,
+            lines(dat.high$xpos, dat.high$ypos,
                   lwd = W.matrix[labels == highs[1], labels == highs[2]] * lwd.multiply[n.plines],
                   col = cols.highlight[n.plines])
       }
@@ -493,23 +713,28 @@ netgraph <- function(x, seq = x$seq,
     ##
     if (!is.null(labels))
       for (i in 1:n)
-        text(pd$xpos.labels[i], pd$ypos.labels[i],
-             labels = pd$labels[i],
+        text(dat.nodes$xpos.labels[i], dat.nodes$ypos.labels[i],
+             labels = dat.nodes$labels[i],
              cex = cex,
-             adj = c(pd$adj1[i], pd$adj2[i]))
+             adj = c(dat.nodes$adj.x[i], dat.nodes$adj.y[i]))
     ##
     ## Print number of treatments
     ##
     if (number.of.studies) {
+      comp.i <- 1
+      ##
       for (i in 1:(n - 1)) {
         for (j in (i + 1):n) {
           if (A.sign[i, j] > 0) {
-            shadowtext((xpos[i] + xpos[j]) / 2,
-                       (ypos[i] + ypos[j]) / 2,
-                       labels = A.matrix[i, j],
+            ##
+            shadowtext(dat.edges$xpos[comp.i],
+                       dat.edges$ypos[comp.i],
+                       labels = dat.edges$n.stud[comp.i],
                        cex = cex.number.of.studies,
                        col = col.number.of.studies,
                        bg = bg.number.of.studies)
+            ##
+            comp.i <- comp.i + 1
           }
         }
       }
@@ -531,27 +756,29 @@ netgraph <- function(x, seq = x$seq,
     ##
     if (!is.null(labels))
       for (i in 1:n)
-        rgl::text3d(pd$xpos.labels[i], pd$ypos.labels[i], pd$zpos.labels[i],
-                    texts = pd$labels[i],
+        rgl::text3d(dat.nodes$xpos.labels[i], dat.nodes$ypos.labels[i],
+                    dat.nodes$zpos.labels[i],
+                    texts = dat.nodes$labels[i],
                     cex = cex,
-                    adj = c(pd$adj1[i], pd$adj2[i]))
+                    adj = c(dat.nodes$adj.x[i], dat.nodes$adj.y[i]))
     ##
     ## Add highlighted comparisons
     ##
     if (!is.null(highlight)) {
       for (high in highlight) {
-        highs <- unlist(strsplit(high, split = highlight.split))
+        highs <- unlist(compsplit(high, split = highlight.split))
         if (length(highs) != 2)
           stop("Wrong format for argument 'highlight' (see helpfile of plotgraph command).")
         ##
-        if (sum(pd$labels %in% highs) != 2)
+        if (sum(dat.nodes$labels %in% highs) != 2)
           stop(paste("Argument 'highlight' must contain two of the following values (separated by \":\"):\n  ",
-                     paste(paste("'", pd$labels, "'", sep = ""),
+                     paste(paste("'", dat.nodes$labels, "'", sep = ""),
                            collapse = " - "), sep = ""))
         ##
-        pdh <- pd[pd$labels %in% highs, ]
+        dat.high <- dat.nodes[dat.nodes$labels %in% highs, ]
         ##
-        rgl::lines3d(pdh$xpos * (1 + 1e-4), pdh$ypos * (1 + 1e-4), pdh$zpos * (1 + 1e-4),
+        rgl::lines3d(dat.high$xpos * (1 + 1e-4), dat.high$ypos * (1 + 1e-4),
+                     dat.high$zpos * (1 + 1e-4),
                      lwd = W.matrix[labels == highs[1], labels == highs[2]],
                      col = col.highlight)
       }
@@ -572,11 +799,11 @@ netgraph <- function(x, seq = x$seq,
           treat2 <- x$treat2[x$studlab %in% multiarm.studies[i]]
           multiarm.labels[[i]] <- sort(unique(c(treat2, treat1)))
           ##
-          pdm <- pd[pd$labels %in% multiarm.labels[[i]], ]
-          if (nrow(pdm) == 0)
-            pdm <- pd[pd$labels %in% multiarm.labels[[i]], ]
-          if (nrow(pdm) == 3)
-            rgl::triangles3d(pdm$xpos, pdm$ypos, pdm$zpos,
+          dat.multi <- dat.nodes[dat.nodes$labels %in% multiarm.labels[[i]], ]
+          if (nrow(dat.multi) == 0)
+            dat.multi <- dat.nodes[dat.nodes$labels %in% multiarm.labels[[i]], ]
+          if (nrow(dat.multi) == 3)
+            rgl::triangles3d(dat.multi$xpos, dat.multi$ypos, dat.multi$zpos,
                              col = col.polygon[i])
           else
             morethan3 <- TRUE
@@ -598,7 +825,30 @@ netgraph <- function(x, seq = x$seq,
       }
     }
   }
+
+
+  is.zero <- function(x) abs(x) < .Machine$double.eps^0.75
+  ##
+  dat.nodes$xpos[is.zero(dat.nodes$xpos)] <- 0
+  dat.nodes$ypos[is.zero(dat.nodes$ypos)] <- 0
+  ##
+  if (!is_2d) {
+    dat.nodes$zpos[is.zero(dat.nodes$zpos)] <- 0
+    ##
+    dat.nodes$xpos.labels <- NULL
+    dat.nodes$ypos.labels <- NULL
+  }
+  else {
+    dat.nodes$xpos.labels[is.zero(dat.nodes$xpos.labels)] <- 0
+    dat.nodes$ypos.labels[is.zero(dat.nodes$ypos.labels)] <- 0
+  }
+  ##
+  dat.nodes$zpos.labels <- NULL
   
   
-  invisible(NULL)
+  dat.edges$xpos[is.zero(dat.edges$xpos)] <- 0
+  dat.edges$ypos[is.zero(dat.edges$ypos)] <- 0
+  
+  
+  invisible(list(nodes = dat.nodes, edges = dat.edges))
 }
