@@ -6,7 +6,9 @@ pairwise <- function(treat,
                      ...) {
   
   
-  if (is.null(data)) data <- sys.frame(sys.parent())
+  null.data <- is.null(data)
+  if (null.data)
+    data <- sys.frame(sys.parent())
   ##
   ## Catch studlab, treat, event, n, mean, sd, time from data:
   ##
@@ -29,7 +31,8 @@ pairwise <- function(treat,
                data, enclos = sys.frame(sys.parent()))
   time <- eval(mf[[match("time", names(mf))]],
                data, enclos = sys.frame(sys.parent()))
-
+  
+  
   args <- list(...)
   nam.args <- names(args)
   
@@ -116,73 +119,127 @@ pairwise <- function(treat,
   
   
   ##
-  ## Transform long format to list format
+  ## Determine whether data is in wide or long arm-based format
   ##
-  treat.list <- list()
-  event.list <- list()
-  n.list     <- list()
-  mean.list  <- list()
-  sd.list    <- list()
-  TE.list    <- list()
-  seTE.list  <- list()
-  time.list  <- list()
+  if (type == "binary")
+    wide.armbased <- is.list(event) & is.list(n)
+  else if (type == "continuous")
+    wide.armbased <- is.list(n) & is.list(mean) & is.list(sd)
+  else if (type == "count")
+    wide.armbased <- is.list(event) & is.list(time)
+  else if (type == "generic")
+    wide.armbased <- is.list(TE) & is.list(seTE)
+  
+  
+  
+  
+  
   ##
-  if (type == "binary") {
-    listformat <- is.list(event) & is.list(n)
-    if (!listformat) {
-      if (is.null(studlab))
-        stop("Argument 'studlab' mandatory if argument 'event' is a vector.")
+  ## Transform long arm-based format to list format
+  ##
+  if (!wide.armbased) {
+    if (is.null(studlab))
+      stop("Argument 'studlab' mandatory if argument 'event' is a vector.")
+    ##
+    studlab <- as.character(studlab)
+    ##
+    treat <- as.character(treat)
+    ##
+    ttab <- table(studlab, treat)
+    n.arms <- apply(ttab, 1, sum)
+    max.arms <- max(n.arms)
+    ##
+    treat.list <- vector("list", max.arms)
+    event.list <- vector("list", max.arms)
+    n.list     <- vector("list", max.arms)
+    mean.list  <- vector("list", max.arms)
+    sd.list    <- vector("list", max.arms)
+    TE.list    <- vector("list", max.arms)
+    seTE.list  <- vector("list", max.arms)
+    time.list  <- vector("list", max.arms)
+    ##
+    if (!null.data)
+      adddata <- vector("list", max.arms)
+    ##
+    if (type == "binary") {
       ##
-      ttab <- table(as.character(studlab), as.character(treat))
-      n.arms <- apply(ttab, 1, sum)
+      ## Generate lists
+      ##
+      tdat <- data.frame(studlab, treat, event, n,
+                         stringsAsFactors = FALSE)
+      ##
+      if (!null.data) {
+        tdat <- cbind(tdat, data)
+        dupl <- duplicated(names(tdat))
+        if (any(dupl))
+          names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
+      }
       ##
-      tdat <- data.frame(studlab, treat, event, n, stringsAsFactors = FALSE)
       tdat <- tdat[order(tdat$studlab, tdat$treat), ]
       ##
       studlab <- names(n.arms)
-      tres <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
+      dat.studlab <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
       ##
-      for (i in 1:max(n.arms)) {
-        tdat.i <- tdat[!duplicated(tdat$studlab), ]
-        tres.i <- merge(tres, tdat.i, by = "studlab", all.x = TRUE)
+      for (i in 1:max.arms) {
+        sel.i <- !duplicated(tdat$studlab)
+        tdat.i <- merge(dat.studlab, tdat[sel.i, ],
+                        by = "studlab", all.x = TRUE)
         ##
-        treat.list[[i]] <- tres.i$treat
-        event.list[[i]] <- tres.i$event
-        n.list[[i]]     <- tres.i$n
+        treat.list[[i]] <- tdat.i$treat
+        event.list[[i]] <- tdat.i$event
+        n.list[[i]]     <- tdat.i$n
         ##
-        tdat <- tdat[duplicated(tdat$studlab), ]
+        tdat.i$event <- NULL
+        tdat.i$n     <- NULL
+        ##
+        if (!null.data)
+          adddata[[i]] <- tdat.i
+        ##
+        tdat <- tdat[!sel.i, ]
       }
       ##
       treat <- treat.list
       event <- event.list
       n     <- n.list
     }
-  }
-  else if (type == "continuous") {
-    listformat <- is.list(n) & is.list(mean) & is.list(sd)
-    if (!listformat) {
-      if (is.null(studlab))
-        stop("Argument 'studlab' mandatory if argument 'mean' is a vector.")
+    ##
+    else if (type == "continuous") {
       ##
-      ttab <- table(as.character(studlab), as.character(treat))
-      n.arms <- apply(ttab, 1, sum)
+      ## Generate lists
+      ##
+      tdat <- data.frame(studlab, treat, n, mean, sd,
+                         stringsAsFactors = FALSE)
+      ##
+      if (!null.data) {
+        tdat <- cbind(tdat, data)
+        dupl <- duplicated(names(tdat))
+        if (any(dupl))
+          names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
+      }
       ##
-      tdat <- data.frame(studlab, treat, n, mean, sd, stringsAsFactors = FALSE)
       tdat <- tdat[order(tdat$studlab, tdat$treat), ]
       ##
       studlab <- names(n.arms)
-      tres <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
+      dat.studlab <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
       ##
-      for (i in 1:max(n.arms)) {
-        tdat.i <- tdat[!duplicated(tdat$studlab), ]
-        tres.i <- merge(tres, tdat.i, by = "studlab", all.x = TRUE)
+      for (i in 1:max.arms) {
+        sel.i <- !duplicated(tdat$studlab)
+        tdat.i <- merge(dat.studlab, tdat[sel.i, ],
+                        by = "studlab", all.x = TRUE)
         ##
-        treat.list[[i]] <- tres.i$treat
-        n.list[[i]]     <- tres.i$n
-        mean.list[[i]]  <- tres.i$mean
-        sd.list[[i]]    <- tres.i$sd
+        treat.list[[i]] <- tdat.i$treat
+        n.list[[i]]     <- tdat.i$n
+        mean.list[[i]]  <- tdat.i$mean
+        sd.list[[i]]    <- tdat.i$sd
         ##
-        tdat <- tdat[duplicated(tdat$studlab), ]
+        tdat.i$n    <- NULL
+        tdat.i$mean <- NULL
+        tdat.i$sd   <- NULL
+        ##
+        if (!null.data)
+          adddata[[i]] <- tdat.i
+        ##
+        tdat <- tdat[!sel.i, ]
       }
       ##
       treat <- treat.list
@@ -190,62 +247,84 @@ pairwise <- function(treat,
       mean  <- mean.list
       sd    <- sd.list
     }
-  }
-  else if (type == "count") {
-    listformat <- is.list(event) & is.list(time)
-    if (!listformat) {
-      if (is.null(studlab))
-        stop("Argument 'studlab' mandatory if argument 'event' is a vector.")
+    ##
+    else if (type == "count") {
       ##
-      ttab <- table(as.character(studlab), as.character(treat))
-      n.arms <- apply(ttab, 1, sum)
+      ## Generate lists
+      ##
+      tdat <- data.frame(studlab, treat, event, time,
+                         stringsAsFactors = FALSE)
+      ##
+      if (!null.data) {
+        tdat <- cbind(tdat, data)
+        dupl <- duplicated(names(tdat))
+        if (any(dupl))
+          names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
+      }
       ##
-      tdat <- data.frame(studlab, treat, event, time, stringsAsFactors = FALSE)
       tdat <- tdat[order(tdat$studlab, tdat$treat), ]
       ##
       studlab <- names(n.arms)
-      tres <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
+      dat.studlab <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
       ##
-      for (i in 1:max(n.arms)) {
-        tdat.i <- tdat[!duplicated(tdat$studlab), ]
-        tres.i <- merge(tres, tdat.i, by = "studlab", all.x = TRUE)
+      for (i in 1:max.arms) {
+        sel.i <- !duplicated(tdat$studlab)
+        tdat.i <- merge(dat.studlab, tdat[sel.i, ],
+                        by = "studlab", all.x = TRUE)
         ##
-        treat.list[[i]] <- tres.i$treat
-        event.list[[i]] <- tres.i$event
-        time.list[[i]]  <- tres.i$time
+        treat.list[[i]] <- tdat.i$treat
+        event.list[[i]] <- tdat.i$event
+        time.list[[i]]  <- tdat.i$time
         ##
-        tdat <- tdat[duplicated(tdat$studlab), ]
+        tdat.i$event <- NULL
+        tdat.i$time  <- NULL
+        ##
+        if (!null.data)
+          adddata[[i]] <- tdat.i
+        ##
+        tdat <- tdat[!sel.i, ]
       }
       ##
       treat <- treat.list
       event <- event.list
       time  <- time.list
     }
-  }
-  else if (type == "generic") {
-    listformat <- is.list(TE) & is.list(seTE)
-    if (!listformat) {
-      if (is.null(studlab))
-        stop("Argument 'studlab' mandatory if argument 'TE' is a vector.")
+    ##
+    else if (type == "generic") {
       ##
-      ttab <- table(as.character(studlab), as.character(treat))
-      n.arms <- apply(ttab, 1, sum)
+      ## Generate lists
+      ##
+      tdat <- data.frame(studlab, treat, TE, seTE,
+                         stringsAsFactors = FALSE)
+      ##
+      if (!null.data) {
+        tdat <- cbind(tdat, data)
+        dupl <- duplicated(names(tdat))
+        if (any(dupl))
+          names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
+      }
       ##
-      tdat <- data.frame(studlab, treat, TE, seTE, stringsAsFactors = FALSE)
       tdat <- tdat[order(tdat$studlab, tdat$treat), ]
       ##
       studlab <- names(n.arms)
-      tres <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
+      dat.studlab <- data.frame(studlab = studlab, stringsAsFactors = FALSE)
       ##
-      for (i in 1:max(n.arms)) {
-        tdat.i <- tdat[!duplicated(tdat$studlab), ]
-        tres.i <- merge(tres, tdat.i, by = "studlab", all.x = TRUE)
+      for (i in 1:max.arms) {
+        sel.i <- !duplicated(tdat$studlab)
+        tdat.i <- merge(dat.studlab, tdat[sel.i, ],
+                        by = "studlab", all.x = TRUE)
         ##
-        treat.list[[i]] <- tres.i$treat
-        TE.list[[i]]    <- tres.i$TE
-        seTE.list[[i]]  <- tres.i$seTE
+        treat.list[[i]] <- tdat.i$treat
+        TE.list[[i]]    <- tdat.i$TE
+        seTE.list[[i]]  <- tdat.i$seTE
         ##
-        tdat <- tdat[duplicated(tdat$studlab), ]
+        tdat.i$TE   <- NULL
+        tdat.i$seTE <- NULL
+        ##
+        if (!null.data)
+          adddata[[i]] <- tdat.i
+        ##
+        tdat <- tdat[!sel.i, ]
       }
       ##
       treat <- treat.list
@@ -255,19 +334,16 @@ pairwise <- function(treat,
   }
   
   
-  
-  
-  
   ##
   ## Check and set study labels
   ##
   if (is.null(studlab))
     studlab <- seq(along = treat[[1]])
   ##
-  if (length(studlab) != length(unique(studlab)))
+  if (length(treat) != 2 && length(studlab) != length(unique(studlab)))
     stop("Study labels must all be distinct.")
   ##
-  levs <- studlab
+  levs <- unique(studlab)
   
   
   narms <- length(treat)
@@ -275,13 +351,75 @@ pairwise <- function(treat,
   
   
   ##
-  ## Auxiliary function to calculate number of arms with zero events
-  ## per study
+  ## Auxiliary functions
   ##
-  sumzero <- function(x) sum(x[!is.na(x)] == 0)
+  sumzero <- function(x)
+    sum(x[!is.na(x)] == 0)
+  ##
+  anytrue <- function(x)
+    any(x == TRUE, na.rm = TRUE)
+
+
+  ##
+  ##
+  ## Generate dataset with variables from original dataset
+  ##
+  ##
+  if (!null.data & !wide.armbased) {
+    names.adddata <- names(adddata[[1]])
+    ##
+    notunique <- matrix(NA,
+                        ncol = length(names.adddata),
+                        nrow = narms * (narms - 1) / 2)
+    colnames(notunique) <- names.adddata
+    ##
+    n.ij <- 0
+    ##
+    for (i in 1:(narms - 1)) {
+      for (j in (i + 1):narms) {
+        n.ij <- n.ij + 1
+        notunique[n.ij, ] <- apply(adddata[[i]] != adddata[[j]], 2, anytrue)
+      }
+    }
+    ##
+    notunique <- apply(notunique, 2, anytrue)
+    ##
+    for (i in 1:(narms - 1)) {
+      for (j in (i + 1):narms) {
+        dat.i <- adddata[[i]]
+        dat.j <- adddata[[j]]
+        ##
+        if (any(!notunique))
+          dat.ij <- dat.i[, names.adddata[!notunique]]
+        else
+          stop("Study label must be unique for single treatment arm.")
+        ##
+        for (nam in names.adddata[notunique]) {
+          dat.ij[, paste(nam, 1, sep = "")] <- adddata[[i]][nam]
+          dat.ij[, paste(nam, 2, sep = "")] <- adddata[[j]][nam]
+        }
+        ##
+        if (i == 1 & j == 2)
+          newdata <- dat.ij
+        else
+          newdata <- rbind(newdata, dat.ij)
+      }
+    }
+    ##
+    names.basic <- c("studlab", "treat1", "treat2")
+    names.newdata <- names(newdata)
+    ##
+    newdata <- newdata[, c(names.basic,
+                           names.newdata[!(names.newdata %in% names.basic)])]
+    newdata <- newdata[!is.na(newdata$treat1) & !is.na(newdata$treat2), ]
+  }
+  
+  
+  
   
   
   if (type == "binary") {
+    ##
     if (length(event) != narms)
       stop("Different length of lists 'treat' and 'event'.")
     if (length(n) != narms)
@@ -344,6 +482,13 @@ pairwise <- function(treat,
                           allstudies = allstudies,
                           stringsAsFactors = FALSE)
         ##
+        if (wide.armbased) {
+          dat <- cbind(dat, data, stringsAsFactors = FALSE)
+          dupl <- duplicated(names(dat))
+          if (any(dupl))
+            names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
+        }
+        ##
         dat <- dat[!(is.na(dat$event1) & is.na(dat$n1)), ]
         dat <- dat[!(is.na(dat$event2) & is.na(dat$n2)), ]
         ##
@@ -445,6 +590,14 @@ pairwise <- function(treat,
                           n1 = n[[i]], mean1 = mean[[i]], sd1 = sd[[i]],
                           n2 = n[[j]], mean2 = mean[[j]], sd2 = sd[[j]],
                           stringsAsFactors = FALSE)
+        ##
+        if (wide.armbased) {
+          dat <- cbind(dat, data, stringsAsFactors = FALSE)
+          dupl <- duplicated(names(dat))
+          if (any(dupl))
+            names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
+        }
+        ##
         dat <- dat[!(is.na(dat$n1) & is.na(dat$mean1) & is.na(dat$sd1)), ]
         dat <- dat[!(is.na(dat$n2) & is.na(dat$mean2) & is.na(dat$sd2)), ]
         ##
@@ -501,6 +654,14 @@ pairwise <- function(treat,
                           TE1 = TE[[i]], seTE1 = seTE[[i]],
                           TE2 = TE[[j]], seTE2 = seTE[[j]],
                           stringsAsFactors = FALSE)
+        ##
+        if (wide.armbased) {
+          dat <- cbind(dat, data, stringsAsFactors = FALSE)
+          dupl <- duplicated(names(dat))
+          if (any(dupl))
+            names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
+        }
+        ##
         dat <- dat[!(is.na(dat$TE1) & is.na(dat$seTE1)), ]
         dat <- dat[!(is.na(dat$TE2) & is.na(dat$seTE2)), ]
         ##
@@ -576,6 +737,14 @@ pairwise <- function(treat,
                           event2 = event[[j]], time2 = time[[j]],
                           incr = incr.study,
                           stringsAsFactors = FALSE)
+        ##
+        if (wide.armbased) {
+          dat <- cbind(dat, data, stringsAsFactors = FALSE)
+          dupl <- duplicated(names(dat))
+          if (any(dupl))
+            names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
+        }
+        ##
         dat <- dat[!(is.na(dat$event1) & is.na(dat$time1)), ]
         dat <- dat[!(is.na(dat$event2) & is.na(dat$time2)), ]
         ##
@@ -605,8 +774,17 @@ pairwise <- function(treat,
       }
     }
   }
-
-
+  ##
+  if (!null.data & !wide.armbased)
+    res <- merge(res, newdata,
+                 by = c("studlab", "treat1", "treat2"),
+                 suffixes = c("",".orig"),
+                 all.x = TRUE)
+  
+  
+  
+  
+  
   ##
   ## Additional checks
   ##
@@ -615,11 +793,11 @@ pairwise <- function(treat,
   ##
   sel.treat <- as.character(res$treat1) == as.character(res$treat2)
   ##
-  if (any(sel.treat)) {
+  if (any(sel.treat))
     stop(paste("Identical treatments for the following studies:\n  ",
-                paste(paste("'", studlab[sel.treat], "'", sep = ""),
-                      collapse = " - "), sep = ""))
-  }
+               paste(paste("'", unique(sort(res$studlab[sel.treat])),
+                           "'", sep = ""),
+                     collapse = " - "), sep = ""))
   ##
   ## b) Studies missing ?
   ##
